package org.apache.juddi.v3.client.config;

import java.util.Collection;
import java.util.Map;
import java.util.Set;

import org.apache.commons.configuration.ConfigurationException;
import org.apache.juddi.v3.annotations.AnnotationProcessor;
import org.apache.juddi.v3.client.transport.Transport;
import org.apache.log4j.Logger;
import org.uddi.api_v3.BusinessService;

public class UDDIClerkManager {
	
	private static UDDIClerkManager manager = null;
	private static Logger log = Logger.getLogger(UDDIClerkManager.class);
    private ClientConfig clientConfig = null;
	
	/**
	 * Singleton.
	 * @throws ConfigurationException 
	 */
	public UDDIClerkManager() throws ConfigurationException {
		super();
		clientConfig = new ClientConfig();
	}
	/**
	 * Stops the clerks.
	 * @throws ConfigurationException 
	 */
	public synchronized static void stop() throws ConfigurationException {
		
		if (manager!=null) {
			log.info("Stopping UDDI Clerks for manager " + manager.clientConfig.getManagerName());
			manager.releaseResources();
			manager=null;
			log.info("UDDI Clerks shutdown completed for manager " + manager.clientConfig.getManagerName());
		} else {
			log.warn("UDDI Clerks where not running for manager " + manager.clientConfig.getManagerName());
		}
	}
	
	private void releaseResources() {
		unRegisterAnnotatedServices();
	}
 	/**
	 * Initializes the UDDI Clerk.
	 * @throws ConfigurationException  
	 */
	public synchronized static void start() throws ConfigurationException {
		if (manager!=null) {
			log.warn("ClerkManager was already started. Going to reload..");
			manager.releaseResources();
			manager.clientConfig.loadManager();
		} else {
			log.info("Starting UDDI Clerks...");
			manager = new UDDIClerkManager();
		}
		if (manager.clientConfig.isRegisterOnStartup()) {
			manager.saveClerkAndNodeInfo();
			manager.registerAnnotatedServices();
			manager.xRegister();
		}
		log.info("Clerks started succesfully for manager " + manager.clientConfig.getManagerName());
 	}
	
	public synchronized static void restart() throws ConfigurationException {
		if (manager==null) {
			log.warn("ClerkManager nor running..");
		} else {
			log.info("Restarting UDDI Clerks...");
			manager.releaseResources();
			manager.clientConfig.loadManager();
			manager = new UDDIClerkManager();
			manager.registerAnnotatedServices();
			manager.xRegister();
			log.info("Clerks restarted succesfully for manager " + manager.clientConfig.getManagerName());
		}
 	}
	/**
	 * Saves the clerk and node info from the uddi.xml to the jUDDI registry.
	 */
	public void saveClerkAndNodeInfo() {
		
		Map<String,UDDIClerk> uddiClerks = clientConfig.getUDDIClerks();
		if (uddiClerks.size() > 0) {
			for (UDDIClerk defaultClerk : uddiClerks.values()) {
				if (Transport.DEFAULT_NODE_NAME.equals(defaultClerk.uddiNode.getName())) {
					for (UDDINode uddiNode : clientConfig.getUDDINodes().values()) {
						defaultClerk.saveNode(uddiNode.getApiNode());
					}
					for (UDDIClerk uddiClerk : clientConfig.getUDDIClerks().values()) {
						defaultClerk.saveClerk(uddiClerk);
					}
					break;
				}
			}
		}	
	}
	
	/**
	 * X-Register services listed in the uddi.xml
	 */
	public void xRegister() {
		log.info("Starting cross registration...");
		//XRegistration of listed businesses
		Set<XRegistration> xBusinessRegistrations = clientConfig.getXBusinessRegistrations();
		for (XRegistration xRegistration : xBusinessRegistrations) {
			xRegistration.xRegisterBusiness();
		}
		//XRegistration of listed serviceBindings
		Set<XRegistration> xServiceBindingRegistrations = clientConfig.getXServiceBindingRegistrations();
		for (XRegistration xRegistration : xServiceBindingRegistrations) {
			xRegistration.xRegisterServiceBinding();
		}
		log.info("Cross registration completed");
	}
	/**
	 * Registers services to UDDI using a clerk, and the uddi.xml
	 * configuration.
	 */
	public void registerAnnotatedServices() {
		Map<String,UDDIClerk> uddiClerks = clientConfig.getUDDIClerks();
		if (uddiClerks.size() > 0) {
			AnnotationProcessor ap = new AnnotationProcessor();
			for (UDDIClerk uddiClerk : uddiClerks.values()) {
				Collection<BusinessService> services = ap.readServiceAnnotations(
						uddiClerk.getClassWithAnnotations(),uddiClerk.getUDDINode().getProperties());
				for (BusinessService businessService : services) {
					log.info("Node=" + uddiClerk.getUDDINode().getApiNode().getName());
					uddiClerk.register(businessService, uddiClerk.getUDDINode().getApiNode());
				}
			}
		}
	}
	/**
	 * Removes the bindings of the services of the annotated classes.
	 */
	public void unRegisterAnnotatedServices() {
		Map<String,UDDIClerk> clerks = clientConfig.getUDDIClerks();
		if (clerks.size() > 0) {
			AnnotationProcessor ap = new AnnotationProcessor();
			for (UDDIClerk clerk : clerks.values()) {
				Collection<BusinessService> services = ap.readServiceAnnotations(
						clerk.getClassWithAnnotations(),clerk.getUDDINode().getProperties());
				for (BusinessService businessService : services) {
					clerk.unRegister(businessService,clerk.getUDDINode().getApiNode());
				}
			}
		}
	}
	
	public static ClientConfig getClientConfig() throws ConfigurationException {
		return getManager().clientConfig;
	}
	
	public static UDDIClerkManager getManager() throws ConfigurationException {
		
		if (manager==null) {
			UDDIClerkManager.start();
		}
		return manager;
	}
	
	
	
	
}

